*************************************************************************************
*														*
*	digital clock for the EASy68k simulator	2007/1/8					*
*														*
*	Just a simple 24 hour digital clock in 68k assembly using the simulated		*
*	hardware in the EASy68k simulator to render the display.				*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	http://www.themotionstore.com/leeedavison/index.html					*
*														*
*	.. or ..												*
*														*
*	http://members.lycos.co.uk/leeedavison/index.html					*
*														*
*	mail : leeedavison@lycos.co.uk								*
*														*
*************************************************************************************

*************************************************************************************
*
* code start. set up the table pointers and initialise the display, this only needs
* to be done the once

	ORG		$1000				* set origin

start
	LEA		LAB_chr(pc),a2		* get pointer to character pattern table
	LEA		LAB_vars(pc),a3		* get pointer to variables
	LEA		LAB_dindex(pc),a4		* get pointer to digit index table
	BSR		LAB_init_disp		* initialise display

* it's the first time through so set all the last time digits to $0A to force an
* update of all the clock digits this time

	LEA		LAB_last(a3),a1		* point to last time (seconds)
	MOVEQ		#5,d7				* six digits to do
	MOVEQ		#$0A,d0			* set for always update digit
LAB_clrl
	MOVE.b	d0,(a1)+			* set digit
	DBRA		d7,LAB_clrl			* loop

* main program loop starts here

LAB_main
	MOVEQ		#8,d0				* get 100ths of a second since midnight
	TRAP		#15

* time in centiseconds since midnight is now in d1. first divide this by 100
* to give the time in seconds. to ensure that there is no overflow of the
* divide instruction the longword time is split into words and each handled
* separately

	MOVEQ		#0,d0				* clear longword
	SWAP		d1				* switch words
	MOVE.w	d1,d0				* copy time high word
	CLR.w		d1				* clear high word

	MOVEQ		#$64,d2			* 100 decimal
	DIVU.w	d2,d0				* divide high word
	SWAP		d0 				* switch words (remainder in low word)
	MOVE.w	d0,d1				* move remainder back
	SWAP		d0 				* switch words back (result in low word)

	SWAP		d1				* switch words
	DIVU.w	d2,d1				* divide low word
	SWAP		d1				* switch words (remainder in low word)
	MOVE.w	d0,d1				* copy seconds high word back
	SWAP		d1				* switch words d1.l, now holds seconds

* now divide the seconds up into the value for each digit. there is no need to
* do this a word at a time as the first divide can never cause an overflow. there
* are at most only 8368 tens of seconds in a day 

	LEA		LAB_curr(a3),a0		* point to current time (seconds)
	MOVEQ		#10,d0			* divide by 10 for seconds
	BSR.s		LAB_digit			* do digit
	MOVEQ		#6,d0				* divide by 6 for minutes
	BSR.s		LAB_digit			* do digit
	MOVEQ		#10,d0			* divide by 10 for minutes
	BSR.s		LAB_digit			* do digit
	MOVEQ		#6,d0				* divide by 6 for hours
	BSR.s		LAB_digit			* do digit
	MOVEQ		#10,d0			* divide by 10 for hours
	BSR.s		LAB_digit			* do digit
	MOVE.b	d1,(a0)			* save 10s of hours digit

* the current time is now divided into digits and saved to memory. each digit is
* compared to the same digit the last time here and if they differ that digit is
* updated. at the first digit match the loop exits as all remaining digits will
* be the same.

	LEA		-5(a0),a0			* reset current time pointer
	LEA		LAB_last(a3),a1		* point to last time (seconds)

	MOVEQ		#5,d7				* six digits to do
LAB_displ
	MOVEQ		#0,d0				* clear longword
	MOVEQ		#0,d1				* clear longword
	MOVE.b	(a0)+,d0			* get new digit
	MOVE.b	(a1),d1			* get last digit

	CMP.b		d0,d1				* compare digits
	BEQ.s		LAB_main			* if no digit change exit loop

	MOVE.b	d0,(a1)+			* else make last digit = new digit

	MOVEQ		#0,d6				* clear longword
	MOVE.b	(a4,d7.w),d6		* get HHMMSS digit index
	MOVE.b	(a2,d0.w),(a5,d6.w)	* copy segment pattern to display

	DBRA		d7,LAB_displ		* loop if more digits to check

	BRA.s		LAB_main			* else return to main loop


*************************************************************************************
*
* extract the next digit value from the time. this is easy on the 68k as the DIVU
* instruction returns both the result of a divide and the remainder. in this case
* the remainder is what we are interested in and is saved to the digit, the divide
* result is just passed back

LAB_digit
	DIVU.w	d0,d1				* divide for next digit
	SWAP		d1				* remainder into low word
	MOVE.b	d1,(a0)+			* save time digit
	CLR.w		d1				* clear remainder
	SWAP 		d1				* return the remaining time
	RTS


*************************************************************************************
*
* initialise the display. the base address of the display is found and copied to a5,
* then each digit is set to '-'

LAB_init_disp
	MOVEQ		#32,d0			* set up hardware
	MOVEQ		#0,d1				* display hardware window
	TRAP		#15				* do I/O function

	MOVEQ		#1,d1				* get display address
	TRAP		#15				* do I/O function

	MOVEA.l	d1,a5				* set display pointer

	MOVEQ		#$0E,d0			* set index to last digit
	MOVEQ		#$40,d1			* set '-' digit
LAB_initl
	MOVE.b	d1,(a5,d0.w)		* save the digit
	SUBQ		#$2,d0			* decrement the index
	BPL.s		LAB_initl			* loop if not all done

	RTS


*************************************************************************************
*
* end of code, start of data

* digit index table. this is a simple way to map the HHMMSS digits as HH-MM-SS

LAB_dindex
	dc.b	$00					* 10's of hours
	dc.b	$02					* hours
	dc.b	$06					* 10's of minutes
	dc.b	$08					* minutes
	dc.b	$0C					* 10's of seconds
	dc.b	$0E					* seconds

*	  00000	Segments patterns. Segments A through G are
*	 5     1	represented by bits 0 through 6 as shown.
*	 5     1	The segment patterns to display the numbers
*	  66666	0 to 9 are made by summing the 2^n value for
*	 4     2	each of the on segments of the number.
*	 4     2
*	  33333

LAB_chr
	dc.b	$3F					* 0	2^0 + 2^1 + 2^2 + 2^3 + 2^4 + 2^5
	dc.b	$06					* 1	2^1 + 2^2
	dc.b	$5B					* 2	2^0 + 2^1 + 2^3 + 2^4 + 2^6
	dc.b	$4F					* 3	2^0 + 2^1 + 2^2 + 2^3 + 2^6
	dc.b	$66					* 4	2^1 + 2^2 + 2^5 + 2^6
	dc.b	$6D					* 5	2^0 + 2^2 + 2^3 + 2^5 + 2^6
	dc.b	$7D					* 6	2^0 + 2^2 + 2^3 + 2^4 + 2^5 + 2^6
	dc.b	$07					* 7	2^0 + 2^1 + 2^2
	dc.b	$7F					* 8	2^0 + 2^1 + 2^2 + 2^3 + 2^4 + 2^5 + 2^6
	dc.b	$6F					* 9	2^0 + 2^1 + 2^2 + 2^3 + 2^5 + 2^6

*************************************************************************************
*
* variables

LAB_vars

	OFFSET	0				* going to use relative addressing

LAB_curr
	ds.b	6					* current time, seconds to 10s hours
LAB_last
	ds.b	6					* last time, seconds to 10s hours


	END	start


*************************************************************************************
